import { createAnalyzer } from '@previewjs/analyzer-react';
import react from '@vitejs/plugin-react';
import path from 'path';
import url from 'url';

const reactImportRegExp = /import (\* as )?React[ ,]/;
function reactImportsPlugin() {
  return {
    name: "previewjs:react-js-imports",
    async transform(code, id) {
      if (id.includes("node_modules") || path.extname(id) !== ".js" || reactImportRegExp.test(code)) {
        return code;
      }
      return `import React from 'react';${code}`;
    }
  };
}

const reactFrameworkPlugin = {
  info: {
    apiVersion: 5,
    name: "@previewjs/plugin-react"
  },
  isCompatible: async (dependencies) => {
    const version = await dependencies["react"]?.readInstalledVersion();
    if (!version) {
      return false;
    }
    const [major, minor] = version.split(".").map((n) => parseInt(n));
    if (isNaN(major) || isNaN(minor)) {
      return false;
    }
    return major >= 17 || major === 16 && minor >= 14;
  },
  async create({ rootDir, reader, logger, dependencies }) {
    const __dirname = url.fileURLToPath(new URL(".", import.meta.url));
    const analyzerPlugin = createAnalyzer({
      rootDir,
      reader,
      logger
    });
    const previewDirPath = path.join(__dirname, "..", "preview");
    return {
      defaultWrapperPath: "__previewjs__/Wrapper.tsx",
      previewDirPath,
      ...analyzerPlugin,
      viteConfig: (configuredPlugins) => {
        const hasReactPlugin = configuredPlugins.find(
          (plugin) => plugin.name.startsWith("vite:react-")
        );
        return {
          resolve: {
            alias: {
              "react-native": "react-native-web"
            }
          },
          plugins: [
            reactImportsPlugin(),
            ...configuredPlugins,
            ...!hasReactPlugin ? [
              // @ts-ignore
              react()
            ] : [],
            {
              name: "previewjs:update-react-import",
              async transform(code, id) {
                if (!id.endsWith("__previewjs_internal__/renderer/index.tsx")) {
                  return;
                }
                const reactVersion = parseInt(
                  await dependencies["react"]?.readInstalledVersion() || "0"
                );
                return code.replace(
                  /__PREVIEWJS_PLUGIN_REACT_IMPORT_PATH__/g,
                  reactVersion >= 18 ? "./render-18" : "./render-16"
                );
              }
            }
          ]
        };
      }
    };
  }
};

export { reactFrameworkPlugin as default };
